import pytest
import torch
from src.modelimpl.torch_dataset import Dataset


class MockSplitData:
    def __init__(self, labels, texts):
        self.labels = labels
        self.texts = texts


def test_dataset():
    # Create a sample SplitData instance
    labels = [0, 1, 1, 0]
    texts = [torch.rand((5,)), torch.rand((5,)), torch.rand((5,)), torch.rand((5,))]
    split_data = MockSplitData(labels, texts)

    # Initialize the Dataset instance
    dataset = Dataset(split_data)

    assert len(dataset) == len(labels)
    assert dataset.classes() == labels

    idx = 2
    assert torch.all(torch.eq(torch.from_numpy(dataset.get_batch_labels(idx)), torch.tensor(labels[idx])))

    assert torch.equal(dataset.get_batch_texts(idx), texts[idx])

    batch_texts, batch_y = dataset[idx]
    assert torch.equal(batch_texts, texts[idx])
    assert torch.tensor(batch_y) == torch.tensor(labels[idx])


def test_dataset_empty_split_data():
    empty_split_data = MockSplitData([], [])

    dataset = Dataset(empty_split_data)

    assert len(dataset) == 0
    assert dataset.classes() == []

    with pytest.raises(IndexError):
        _ = dataset[0]
